/***** ms.c     ************************************************
*
*       Generates samples of gametes ( theta given or fixed number
*						of segregating sites.)
*	Usage is shown by typing ms without arguments.
        usage: ms nsam howmany  -t  theta  [options]
		or
	       ms nsam howmany -s segsites  [options]

	   nsam is the number of gametes per sample.
	   howmany is the number of samples to produce.
	   With -t the numbers of segregating sites will randomly vary
		from one sample to the next.
	   with -s segsites,  the number of segregating sites will be
		segsites in each sample.

           Other options: See msdoc.pdf or after downloading and compiling, type ms<CR>.


*	  Arguments of the options are explained here:

           npop:  Number of subpopulations which make up the total population
           ni:  the sample size from the i th subpopulation (all must be
		specified.) The output will have the gametes in order such that
		the first n1 gametes are from the first island, the next n2 are
		from the second island, etc.
           nsites: number of sites between which recombination can occur.
           theta: 4No times the neutral mutation rate
           rho: recombination rate between ends of segment times 4No
	   f: ratio of conversion rate to recombination rate. (Wiuf and Hein model.)
	   track_len:  mean length of conversion track in units of sites.  The
		       total number of sites is nsites, specified with the -r option.
           mig_rate: migration rate: the fraction of each subpop made up of
                 migrants times 4No.
           howmany: howmany samples to generate.

	Note:  In the above definition, No is the total diploid population if
		npop is one, otherwise, No is the diploid population size of each
		subpopulation.
	A seed file called "seedms" will be created  if it doesn't exist. The
		seed(s) in this file will be modified by the program.
		So subsequent runs
		will produce new output.  The initial contents of seedms will be
		printed on the second line of the output.
        Output consists of one line with the command line arguments and one
	 	line with the seed(s).
		The samples appear sequentially following that line.
		Each sample begins with "//", then the number of segregating sites, the positions
		of the segregating sites (on a scale of 0.0 - 1.0). On the following
		lines are the sampled gametes, with mutants alleles represented as
		ones and ancestral alleles as zeros.
	To compile:  cc -o ms  ms.c  streec.c  rand1.c -lm
		or:  cc -o ms ms.c streec.c rand2.c -lm
	 (Of course, gcc would be used instead of cc on some machines.  And -O3 or
		some other optimization switches might be usefully employed with some
		compilers.) ( rand1.c uses drand48(), whereas rand2.c uses rand() ).

*
*   Modifications made to combine ms and mss on 25 Feb 2001
*	Modifications to command line options to use switches  25 Feb 2001
*	Modifications to add // before each sample  25 Feb 2001
	Modifications to add gene conversion 5 Mar 2001
	Added demographic options -d  13 Mar 2001
	Changed ran1() to use rand(). Changed seed i/o to accomodate this change. 20 April.
	Changed cleftr() to check for zero rand() .13 June 2001
	Move seed stuff to subroutine seedit()  11 July 2001
	Modified streec.c to handle zero length demographic intervals 9 Aug 2001
	Corrected problem with negative growth rates (Thanks to D. Posada and C. Wiuf) 13 May 2002
	Changed sample_stats.c to output thetah - pi rather than pi - thetah.  March 8 2003.
	Changed many command line options, allowing arbitrary migration matrix, and subpopulation
	   sizes.  Also allows parameters to come from a file. Option to output trees.  Option to
	   split and join subpopulations.   March 8, 2003. (Old versions saved in msold.tar ).
	!!! Fixed bug in -en option.  Earlier versions may have produced garbage when -en ... used. 9 Dec 2003
	Fixed bug which resulted in incorrect results for the case where
             rho = 0.0 and gene conversion rate > 0.0. This case was not handled
	    correctly in early versions of the program. 5 Apr 2004.  (Thanks to
	    Vincent Plagnol for pointing out this problem.)
	Fixed bug in prtree().  Earlier versions may have produced garbage when the -T option was used.
		 1 Jul 2004.
	Fixed bug in -e. options that caused problems with -f option  13 Aug 2004.
	Fixed bug in -es option, which was a problem when used with -eG. (Thanks again to V. Plagnol.) 6 Nov. 2004
	Added -F option:  -F minfreq  produces output with sites with minor allele freq < minfreq filtered out.  11 Nov. 2004.
	Fixed bug in streec.c (isseg() ).  Bug caused segmentation fault, crash on some machines. (Thanks
	    to Melissa Jane Todd-Hubisz for finding and reporting this bug.)
	Added -seeds option 4 Nov 2006
	Added "tbs" arguments feature 4 Nov 2006
	Added -L option.  10 May 2007
	Changed -ej option to set Mki = 0 pastward of the ej event.  See msdoc.pdf.  May 19 2007.
	fixed bug with memory allocation when using -I option. This caused problems expecially on Windows
          machines.  Thanks to several people, including Vitor Sousa and Stephane De Mita for help on this one.
          Oct. 17, 2007.
     Modified pickb() and pickbmf() to eliminate rare occurrence of fixed mutations Thanks to J. Liechty and K. Thornton. 4 Feb 2010.
	Added -p n  switch to allow position output to be higher precision.  10 Nov 2012.
     Changed spot from int to long in the function re().  29 July 2013.  ( Thanks to Yuri D'Elia for this suggestion.)
      Changed function definitions, and misc other things to comply with c99
     requirements.  4 Mar 2014.
***************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include "ms.h"

#define SITESINC 10

unsigned maxsites = SITESINC ;


struct node{
	int abv;
	int ndes;
	float time;
	int ncs; //number of descendents
	int masked; // have leaf masked or not. 0 means not masked, 1 means masked

	};

struct segl {
	int beg;
	struct node *ptree;
	int next;
	};

double *posit ;
double segfac ;
int count, ntbs, nseeds ;
struct params pars ;

    int
main(int argc, char *argv[])
{
	int i, k, howmany, segsites, ton, dataCount;
	char **list, **cmatrix(), **tbsparamstrs;
	FILE *pf, *fopen() ;
	double probss, tmrca, ttot ;
	void seedit( const char * ) ;
	void getpars( int argc, char *argv[], int *howmany )  ;
	int gensam( char **list, double *probss, double *ptmrca, double *pttot ) ;
	int gensamWithQM( char **list, double *probss, double *ptmrca, double *pttot, int *qmRow, int *qmCol, int queMarLen) ;

	ntbs = 0 ;   /* these next few lines are for reading in parameters from a file (for each sample) */
	tbsparamstrs = (char **)malloc( argc*sizeof(char *) ) ;

	for( i=0; i<argc; i++) printf("%s ",argv[i]);
	for( i =0; i<argc; i++) tbsparamstrs[i] = (char *)malloc(30*sizeof(char) ) ;
	for( i = 1; i<argc ; i++)
			if( strcmp( argv[i],"tbs") == 0 )  argv[i] = tbsparamstrs[ ntbs++] ;

	count=0;

	if( ntbs > 0 )  for( k=0; k<ntbs; k++)  scanf(" %s", tbsparamstrs[k] );
	getpars( argc, argv, &howmany) ;   /* results are stored in global variable, pars */

	if( !pars.commandlineseedflag ) seedit( "s");
	pf = stdout ;

	if( pars.mp.segsitesin ==  0 ) {
	    if((pars.mfs.singlton == 0) && (pars.mfs.doubleton == 0) && (pars.mfs.otherton == 0)) { // not have mfs -Y
            list = cmatrix(pars.cp.nsam, maxsites+1);
            posit = (double *)malloc( (unsigned)( maxsites*sizeof( double)) ) ;
        } else {
            ton = pars.mfs.doubleton + pars.mfs.singlton + pars.mfs.otherton;
            list = cmatrix(pars.cp.nsam, ton+1);

            posit = (double *)malloc( (unsigned)( ton*sizeof( double)) ) ;
            if( pars.mp.theta > 0.0 ){
                segfac = 1.0 ;
                for(  i= ton; i > 1; i--) segfac *= i ;
            }
        }
	} else {
	     list = cmatrix(pars.cp.nsam, pars.mp.segsitesin+1 ) ;
	     posit = (double *)malloc( (unsigned)( pars.mp.segsitesin*sizeof( double)) ) ;
	     if( pars.mp.theta > 0.0 ){
		    segfac = 1.0 ;
		    for(  i= pars.mp.segsitesin; i > 1; i--) segfac *= i ;
		 }
	}


    int qmRow[pars.mfs.mdSum]; // question marks row index
    int qmCol[pars.mfs.mdSum]; // question marks column index

    if(pars.mfs.mdSum > 0) {  // have question marks


        for(dataCount=0;dataCount<pars.mfs.mdSum;dataCount++) {
            int qmTmp = pars.mfs.mdArr[dataCount];
            //fprintf(pf,"markIndex: %d\n", qmTmp);
            qmRow[dataCount] = qmTmp%pars.cp.nsam;
            qmCol[dataCount] = qmTmp/pars.cp.nsam;
        }
    }

    while( howmany-count++ ) {
	   if( (ntbs > 0) && (count >1 ) ){
	         for( k=0; k<ntbs; k++){
			    if( scanf(" %s", tbsparamstrs[k]) == EOF ){
			       if( !pars.commandlineseedflag ) seedit( "end" );
				   exit(0);
				}
			 }
			 getpars( argc, argv, &howmany) ;
	   }

       fprintf(pf,"\n//");
	   if( ntbs >0 ){
			for(k=0; k< ntbs; k++) printf("\t%s", tbsparamstrs[k] ) ;
       }
		printf("\n");

        if(pars.mfs.mdSum > 0) {  // have question marks
            segsites = gensamWithQM( list, &probss, &tmrca, &ttot, qmRow, qmCol, pars.mfs.mdSum) ;
        } else {
            segsites = gensam( list, &probss, &tmrca, &ttot ) ;
        }

  		if( pars.mp.timeflag ) fprintf(pf,"time:\t%lf\t%lf\n",tmrca, ttot ) ;
        if( (segsites > 0 ) || ( pars.mp.theta > 0.0 ) ) {
   	       if( (pars.mp.segsitesin > 0 ) && ( pars.mp.theta > 0.0 ))
		       fprintf(pf,"prob: %g\n", probss ) ;
           fprintf(pf,"segsites: %d\n",segsites);
		   if( segsites > 0 )	fprintf(pf,"positions: ");
		   for( i=0; i<segsites; i++)
              fprintf(pf,"%6.*lf ", pars.output_precision,posit[i] );
           fprintf(pf,"\n");
	       if( segsites > 0 )
	          for(i=0;i<pars.cp.nsam; i++) { fprintf(pf,"%s\n", list[i] ); }
	    }
    }
	if( !pars.commandlineseedflag ) seedit( "end" );

}



	int
gensam( char **list, double *pprobss, double *ptmrca, double *pttot )
{
	int nsegs, h, i, k, j, seg, ns, start, end, len, segsit ;
	struct segl *seglst, *segtre_mig(struct c_params *p, int *nsegs ) ; /* used to be: [MAXSEG];  */
	double nsinv,  tseg, tt, ttime(struct node *, int nsam), ttimemf(struct node *, int nsam, int mfreq) ;
	double *pk;
	int *ss;

    //stt is singleton total time of all trees, dtt is doubleton total time of all trees
	//ott is otherton total time of all trees
	double stt, dtt, ott;
	// spk[k] stores total time of total tips in each segment tree, 0=<k<nsegs
	// dpk[k] stores total time of total nodes (which has two desendents) in each segment tree, 0=<k<nsegs
	// opk[k] stores total time of total nodes (which has >= three desendents) in each segment tree, 0=<k<nsegs
	double *spk, *dpk, *opk;
	int *sdouton, *ssinton, *sothton;
	int douton, sinton, othton;

	int segsitesin,nsites,mhlen;
	double theta, es ;
	int nsam, mfreq ;
	void prtree( struct node *ptree, int nsam);


    int make_gametes(int nsam, int mfreq, double *posit, long int nsites, struct node *ptree, double tt, int newsites, int ns, char **list );
    int make_MFSgametes(int nsam, int mfreq, double *posit, long int nsites, struct node *ptree, double sintontt, double doutontt, double othtontt, int stonnewsites, int dtonnewsites, int otonnewsites, int ns, char **list, int loop);

 	void ndes_setup( struct node *, int nsam );

    int totalSeg;
    void sdottime(struct node *, int nsam, double* sdot, int unfo);
    void sdottimemf(struct node *, int nsam, double* sdot, int mfreq, int unfo);
    double sdotime[3];

	nsites = pars.cp.nsites ;
	nsinv = 1./nsites;
	seglst = segtre_mig(&(pars.cp),  &nsegs ) ;

	nsam = pars.cp.nsam;
	segsitesin = pars.mp.segsitesin ;
	theta = pars.mp.theta ;
	mfreq = pars.mp.mfreq ;
    mhlen = pars.mfs.mhlength;

	if( pars.mp.treeflag ) {
	  	ns = 0 ;
	    for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
	      if( (pars.cp.r > 0.0 ) || (pars.cp.f > 0.0) ){
		     end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
		     start = seglst[seg].beg ;
		     len = end - start + 1 ;
		     fprintf(stdout,"[%d]", len);
	      }
	      prtree( seglst[seg].ptree, nsam ) ;
	      if( (segsitesin == 0) && ( theta == 0.0 ) && ( pars.mp.timeflag == 0 ) )
	  	      free(seglst[seg].ptree) ;
	    }
	}

	if( pars.mp.timeflag ) {
      tt = 0.0 ;
	  for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
		if( mfreq > 1 ) ndes_setup( seglst[seg].ptree, nsam );
		end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
		start = seglst[seg].beg ;
		if( (nsegs==1) || ( ( start <= nsites/2) && ( end >= nsites/2 ) ) )
		  *ptmrca = (seglst[seg].ptree + 2*nsam-2) -> time ;
		len = end - start + 1 ;
		tseg = len/(double)nsites ;
		if( mfreq == 1 ) tt += ttime(seglst[seg].ptree,nsam)*tseg ;
		else tt += ttimemf(seglst[seg].ptree,nsam, mfreq)*tseg ;
		if( (segsitesin == 0) && ( theta == 0.0 )  )
	  	      free(seglst[seg].ptree) ;
	    }
		*pttot = tt ;
	 }

	 if(segsitesin == 0) {
        if((theta > 0.0) && ((pars.mfs.singlton == 0) && (pars.mfs.doubleton == 0) && (pars.mfs.otherton == 0))) {
            ns = 0 ;
            for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
                if( mfreq > 1 ) ndes_setup( seglst[seg].ptree, nsam );
                end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
                start = seglst[seg].beg ;
                len = end - start + 1 ;
                tseg = len*(theta/nsites) ;
                if( mfreq == 1) tt = ttime(seglst[seg].ptree, nsam);
                        else tt = ttimemf(seglst[seg].ptree, nsam, mfreq );
                segsit = poisso( tseg*tt );
                if( (segsit + ns) >= maxsites ) {
                    maxsites = segsit + ns + SITESINC ;
                    posit = (double *)realloc(posit, maxsites*sizeof(double) ) ;
                      biggerlist(nsam, list) ;
                }
		  locate(segsit,start*nsinv, len*nsinv,posit+ns);
                make_gametes(nsam,mfreq, posit, mhlen, seglst[seg].ptree,tt, segsit, ns, list );
                free(seglst[seg].ptree) ;
                
                ns += segsit;
            }
        } else if((pars.mfs.singlton > 0) || (pars.mfs.doubleton > 0) || (pars.mfs.otherton > 0)) {

             douton = pars.mfs.doubleton;
             sinton = pars.mfs.singlton;
             othton = pars.mfs.otherton;

             // for mfs
             spk = (double *)malloc((unsigned)(nsegs*sizeof(double)));
             dpk = (double *)malloc((unsigned)(nsegs*sizeof(double)));
             opk = (double *)malloc((unsigned)(nsegs*sizeof(double)));

             sdouton = (int *)malloc((unsigned)(nsegs*sizeof(int)));
             ssinton = (int *)malloc((unsigned)(nsegs*sizeof(int)));
             sothton = (int *)malloc((unsigned)(nsegs*sizeof(int)));

             if( (spk==NULL) || (dpk==NULL) || (opk==NULL) || (ssinton==NULL) || (sdouton==NULL) || (sothton==NULL)) perror("malloc error. gensam.2");

             stt = 0.0;
             dtt = 0.0;
             ott = 0.0;

             for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {

                if( mfreq > 1 ) ndes_setup( seglst[seg].ptree, nsam );
                end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
                start = seglst[seg].beg ;
                len = end - start + 1 ;
                tseg = len/(double)nsites ;

                if(mfreq == 1) {
                    sdottime(seglst[seg].ptree,nsam, sdotime, pars.mfs.unfolded);
                } else {
                    sdottimemf(seglst[seg].ptree,nsam, sdotime, mfreq, pars.mfs.unfolded);
                }
               // printf("stime:%f, dtime:%f, xtime:%f\n", sdotime[0], sdotime[1], sdotime[2] );
                spk[k] = sdotime[0]*tseg;
                dpk[k] = sdotime[1]*tseg;
                opk[k] = sdotime[2]*tseg;
                stt += spk[k];
                dtt += dpk[k];
                ott += opk[k];


              }

              if( theta > 0.0 ) {
                es = theta * (stt+dtt+ott);
                *pprobss = exp( -es )*pow( es, (double)(douton+sinton+othton)) / segfac ;
              }

              //for singlton
	          if(stt > 0.0) {
                for (k=0;k<nsegs;k++) spk[k] /= stt;
                //init ssinton, which stores each singleton number in each segment tree
                mnmial(sinton,nsegs,spk,ssinton);
              } else {
                 for( k=0; k<nsegs; k++) ssinton[k] = 0 ;
              }

              //for doubleton
              if(dtt > 0.0) {
                for (k=0;k<nsegs;k++) dpk[k] /= dtt;
                //init douton, which stores each doubleton number in each segment tree
                mnmial(douton,nsegs,dpk,sdouton);
              } else {
                for( k=0; k<nsegs; k++) sdouton[k] = 0 ;
              }

              //for otherton
              if(ott > 0.0) {
                for (k=0;k<nsegs;k++) opk[k] /= ott;
                mnmial(othton,nsegs,opk,sothton);
              } else {
                for( k=0; k<nsegs; k++) sothton[k] = 0 ;
              }

              ns = 0 ;
              for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
                 end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
                 start = seglst[seg].beg ;
                 len = end - start + 1 ;
                 tseg = len/(double)nsites;
                 totalSeg = ssinton[k] + sdouton[k] + sothton[k];

                 locate(totalSeg, start*nsinv, len*nsinv, posit+ns);

                 make_MFSgametes(nsam, mfreq, posit, mhlen, seglst[seg].ptree, stt*spk[k]/tseg, dtt*dpk[k]/tseg, ott*opk[k]/tseg, ssinton[k], sdouton[k], sothton[k], ns, list, 0);

                 ns = ns + totalSeg;
                 free(seglst[seg].ptree) ;
              }

              free(spk);
              free(dpk);
              free(opk);
              free(ssinton);
              free(sdouton);
              free(sothton);
        }
    } else if( segsitesin > 0 ) {

        pk = (double *)malloc((unsigned)(nsegs*sizeof(double)));
        ss = (int *)malloc((unsigned)(nsegs*sizeof(int)));
        if( (pk==NULL) || (ss==NULL) ) perror("malloc error. gensam.2");


        tt = 0.0 ;
      for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
        if( mfreq > 1 ) ndes_setup( seglst[seg].ptree, nsam );
        end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
        start = seglst[seg].beg ;
        len = end - start + 1 ;
        tseg = len/(double)nsites ;
               if( mfreq == 1 ) pk[k] = ttime(seglst[seg].ptree,nsam)*tseg ;
               else pk[k] = ttimemf(seglst[seg].ptree,nsam, mfreq)*tseg ;
                 tt += pk[k] ;
      }
	  if( theta > 0.0 ) {
	    es = theta * tt ;
	    *pprobss = exp( -es )*pow( es, (double) segsitesin) / segfac ;
	  }
	  if( tt > 0.0 ) {
          for (k=0;k<nsegs;k++) pk[k] /= tt ;
          mnmial(segsitesin,nsegs,pk,ss);
	  }
	  else
            for( k=0; k<nsegs; k++) ss[k] = 0 ;
	  ns = 0 ;
	  for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
		 end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
		 start = seglst[seg].beg ;
		 len = end - start + 1 ;
		 tseg = len/(double)nsites;
		 locate(ss[k],start*nsinv, len*nsinv,posit+ns);
		 make_gametes(nsam,mfreq, posit, mhlen, seglst[seg].ptree,tt*pk[k]/tseg, ss[k], ns, list);
		 free(seglst[seg].ptree) ;
		

		 ns += ss[k] ;
	  }
	  free(pk);
	  free(ss);

    }

	for(i=0;i<nsam;i++) list[i][ns] = '\0' ;
	return( ns ) ;
}


int
gensamWithQM( char **list, double *pprobss, double *ptmrca, double *pttot, int *qmRow, int *qmCol, int queMarLen )
{
	int nsegs, h, i, k, j, seg, ns, start, end, len, segsit ;
	struct segl *seglst, *segtre_mig(struct c_params *p, int *nsegs ) ; /* used to be: [MAXSEG];  */
	double nsinv,  tseg, tt, ttime(struct node *, int nsam), ttimemf(struct node *, int nsam, int mfreq) ;

	double stt, dtt, ott;

   	double *pk;

	double *spk, *dpk, *opk;

	int *ss;
	int *sdouton, *ssinton, *sothton;
    int douton, sinton, othton;
	int segsitesin,nsites, mhlen;
	double theta, es ;
	int nsam, mfreq ;
	void prtree( struct node *ptree, int nsam);
	int make_MFSgametesQM(int nsam, double *posit, long int mhlen, struct node *ptree, double* sdoTemTime, int* sdoTemSite, int ns, char **list, int* currTreRow, int* currTreCol, int currTreLen);

 	void ndes_setup( struct node *, int nsam );

    int totalSeg;
    void sdottime(struct node *, int nsam, double* sdot, int unfo);
    void sdottimemf(struct node *, int nsam, double* sdot, int mfreq, int unfo);
    double sdotime[3];
    int currTreQM, qmCount, currCount;

	nsites = pars.cp.nsites ;
	nsinv = 1./nsites;


    seglst = segtre_mig(&(pars.cp),  &nsegs ) ;

	nsam = pars.cp.nsam;
	segsitesin = pars.mp.segsitesin ;
	theta = pars.mp.theta ;
	mfreq = pars.mp.mfreq ;
    mhlen = pars.mfs.mhlength;

    douton = pars.mfs.doubleton;
    sinton = pars.mfs.singlton;
    othton = pars.mfs.otherton;

    // for mfs
    spk = (double *)malloc((unsigned)(nsegs*sizeof(double)));
    dpk = (double *)malloc((unsigned)(nsegs*sizeof(double)));
    opk = (double *)malloc((unsigned)(nsegs*sizeof(double)));

    sdouton = (int *)malloc((unsigned)(nsegs*sizeof(int)));
    ssinton = (int *)malloc((unsigned)(nsegs*sizeof(int)));
    sothton = (int *)malloc((unsigned)(nsegs*sizeof(int)));

    if( (spk==NULL) || (dpk==NULL) || (opk==NULL) || (ssinton==NULL) || (sdouton==NULL) || (sothton==NULL)) perror("malloc error. gensam.2");

     stt = 0.0;
     dtt = 0.0;
     ott = 0.0;

     for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {

        if( mfreq > 1 ) ndes_setup( seglst[seg].ptree, nsam );
        end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
        start = seglst[seg].beg ;
        len = end - start + 1 ;
        tseg = len/(double)nsites ;

        if(mfreq == 1) {
            sdottime(seglst[seg].ptree,nsam, sdotime, pars.mfs.unfolded);
        } else {
            sdottimemf(seglst[seg].ptree,nsam, sdotime, mfreq, pars.mfs.unfolded);
        }
       // printf("stime:%f, dtime:%f, xtime:%f\n", sdotime[0], sdotime[1], sdotime[2] );
        spk[k] = sdotime[0]*tseg;
        dpk[k] = sdotime[1]*tseg;
        opk[k] = sdotime[2]*tseg;
        stt += spk[k];
        dtt += dpk[k];
        ott += opk[k];
      }

      //for singlton
      if(stt > 0.0) {
        for (k=0;k<nsegs;k++) spk[k] /= stt;
        //init ssinton, which stores each singleton number in each segment tree
        mnmial(sinton,nsegs,spk,ssinton);
      } else {
         for( k=0; k<nsegs; k++) ssinton[k] = 0 ;
      }

      //for doubleton
      if(dtt > 0.0) {
        for (k=0;k<nsegs;k++) dpk[k] /= dtt;
        //init douton, which stores each doubleton number in each segment tree
        mnmial(douton,nsegs,dpk,sdouton);
      } else {
        for( k=0; k<nsegs; k++) sdouton[k] = 0 ;
      }

      //for otherton
      if(ott > 0.0) {
        for (k=0;k<nsegs;k++) opk[k] /= ott;
        mnmial(othton,nsegs,opk,sothton);
      } else {
        for( k=0; k<nsegs; k++) sothton[k] = 0 ;
      }

      ns = 0 ;
      double sdoTemTime[3] = {0.0, 0.0, 0.0};
      int sdoTemSite[3] = {0, 0, 0};

      for( seg=0, k=0; k<nsegs; seg=seglst[seg].next, k++) {
         end = ( k<nsegs-1 ? seglst[seglst[seg].next].beg -1 : nsites-1 );
         start = seglst[seg].beg ;
         len = end - start + 1 ;
         tseg = len/(double)nsites;
         totalSeg = ssinton[k] + sdouton[k] + sothton[k];


         currTreQM = 0;
         for(qmCount=0;qmCount<queMarLen;qmCount++) {
            if((qmCol[qmCount] >= ns) && (qmCol[qmCount] < (ns+totalSeg))) {
                currTreQM++;
            }
         }
         int currTreRow[currTreQM];
         int currTreCol[currTreQM];
         currCount = 0;
         for(qmCount=0;qmCount<queMarLen;qmCount++) {
            if((qmCol[qmCount] >= ns) && (qmCol[qmCount] < (ns+totalSeg))) {
                currTreRow[currCount] = qmRow[qmCount];
                currTreCol[currCount] = qmCol[qmCount];
                currCount++;
            }
         }


         locate(totalSeg, start*nsinv, len*nsinv, posit+ns);

         sdoTemTime[0] = stt*spk[k]/tseg;
         sdoTemTime[1] = dtt*dpk[k]/tseg;
         sdoTemTime[2] = ott*opk[k]/tseg;

         sdoTemSite[0] = ssinton[k];
         sdoTemSite[1] = sdouton[k];
         sdoTemSite[2] = sothton[k];

         make_MFSgametesQM(nsam, posit, mhlen, seglst[seg].ptree, sdoTemTime, sdoTemSite, ns, list, currTreRow, currTreCol, currTreQM);

         ns = ns + totalSeg;
         free(seglst[seg].ptree) ;
      }

      free(spk);
      free(dpk);
      free(opk);
      free(ssinton);
      free(sdouton);
      free(sothton);


	for(i=0;i<nsam;i++) list[i][ns] = '\0' ;

	return( ns ) ;
}

	void
ndes_setup(struct node *ptree, int nsam )
{
	int i ;

	for( i=0; i<nsam; i++) (ptree+i)->ndes = 1 ;
	for( i = nsam; i< 2*nsam -1; i++) (ptree+i)->ndes = 0 ;
	for( i= 0; i< 2*nsam -2 ; i++)  (ptree+((ptree+i)->abv))->ndes += (ptree+i)->ndes ;

}

	void
biggerlist(int nsam,  char **list )
{
	int i;

/*  fprintf(stderr,"maxsites: %d\n",maxsites);  */
	for( i=0; i<nsam; i++){
	   list[i] = (char *)realloc( list[i],maxsites*sizeof(char) ) ;
	   if( list[i] == NULL ) perror( "realloc error. bigger");
	   }
}



/* allocates space for gametes (character strings) */
	char **
cmatrix(nsam,len)
	int nsam, len;
{
	int i;
	char **m;

	if( ! ( m = (char **) malloc( (unsigned) nsam*sizeof( char* ) ) ) )
	   perror("alloc error in cmatrix") ;
	for( i=0; i<nsam; i++) {
		if( ! ( m[i] = (char *) malloc( (unsigned) len*sizeof( char ) )))
			perror("alloc error in cmatric. 2");
		}
	return( m );
}



	void
locate(int n,double beg, double len,double *ptr)
{
	int i;

	ordran(n,ptr);
	for(i=0; i<n; i++)
		ptr[i] = beg + ptr[i]*len ;

}

int NSEEDS = 3 ;

  void
getpars(int argc, char *argv[], int *phowmany )
{
	int arg, i, j, sum , pop , argstart, npop , npop2, pop2,mdcc ;
	double migr, mij, psize, palpha ;
	void addtoelist( struct devent *pt, struct devent *elist );
	void argcheck( int arg, int argc, char ** ) ;
	int commandlineseed( char ** ) ;
	void free_eventlist( struct devent *pt, int npop );
	struct devent *ptemp , *pt ;
	FILE *pf ;
	char ch3 ;


  if( count == 0 ) {
	if( argc < 4 ){ fprintf(stderr,"Too few command line arguments\n"); usage();}
	pars.cp.nsam = atoi( argv[1] );
	if( pars.cp.nsam <= 0 ) { fprintf(stderr,"First argument error. nsam <= 0. \n"); usage();}
	*phowmany = atoi( argv[2] );
	if( *phowmany  <= 0 ) { fprintf(stderr,"Second argument error. howmany <= 0. \n"); usage();}
	pars.commandlineseedflag = 0 ;
	  pars.output_precision = 4 ;
	pars.cp.r = pars.mp.theta =  pars.cp.f = 0.0 ;
	pars.cp.track_len = 0. ;
	pars.cp.npop = npop = 1 ;
	pars.cp.mig_mat = (double **)malloc( (unsigned) sizeof( double *) );
	pars.cp.mig_mat[0] = (double *)malloc( (unsigned)sizeof(double ));
	pars.cp.mig_mat[0][0] =  0.0 ;
	pars.mp.segsitesin = 0 ;

    //init added arguments
	pars.mfs.unfolded = 1;
    pars.mfs.multipleHit = 0;
    pars.mfs.mhlength = 0;
    pars.mfs.singlton = 0;
    pars.mfs.doubleton = 0;
    pars.mfs.otherton = 0;
    pars.mfs.mdSum = 0;


	pars.mp.treeflag = 0 ;
 	pars.mp.timeflag = 0 ;
       pars.mp.mfreq = 1 ;
	pars.cp.config = (int *) malloc( (unsigned)(( pars.cp.npop +1 ) *sizeof( int)) );
	(pars.cp.config)[0] = pars.cp.nsam ;
	pars.cp.size= (double *) malloc( (unsigned)( pars.cp.npop *sizeof( double )) );
	(pars.cp.size)[0] = 1.0  ;
	pars.cp.alphag = (double *) malloc( (unsigned)(( pars.cp.npop ) *sizeof( double )) );
	(pars.cp.alphag)[0] = 0.0  ;
	pars.cp.nsites = 2 ;
  }
  else{
	npop = pars.cp.npop ;
	free_eventlist( pars.cp.deventlist, npop );
  }
  	pars.cp.deventlist = NULL ;

	arg = 3 ;

	while( arg < argc ){
		if( argv[arg][0] != '-' ) { fprintf(stderr," argument should be -%s ?\n", argv[arg]); usage();}
		switch ( argv[arg][1] ){
			case 'f' :
				if( ntbs > 0 ) { fprintf(stderr," can't use tbs args and -f option.\n"); exit(1); }
				arg++;
				argcheck( arg, argc, argv);
				pf = fopen( argv[arg], "r" ) ;
				if( pf == NULL ) {fprintf(stderr," no parameter file %s\n", argv[arg] ); exit(0);}
				arg++;
				argc++ ;
				argv = (char **)malloc(  (unsigned)(argc+1)*sizeof( char *) ) ;
				argv[arg] = (char *)malloc( (unsigned)(20*sizeof( char )) ) ;
				argstart = arg ;
				while( fscanf(pf," %s", argv[arg]) != EOF ) {
					arg++;
					argc++;
					argv = (char **)realloc( argv, (unsigned)argc*sizeof( char*) ) ;
				        argv[arg] = (char *)malloc( (unsigned)(20*sizeof( char )) ) ;
					}
				fclose(pf);
				argc--;
				arg = argstart ;
				break;
			case 'r' :
				arg++;
				argcheck( arg, argc, argv);
				pars.cp.r = atof(  argv[arg++] );
				argcheck( arg, argc, argv);
				pars.cp.nsites = atoi( argv[arg++]);
				if( pars.cp.nsites <2 ){
					fprintf(stderr,"with -r option must specify both rec_rate and nsites>1\n");
					usage();
					}
				break;
			case 'p' :
				arg++;
				argcheck(arg,argc,argv);
				pars.output_precision = atoi( argv[arg++] ) ;
				break;
			case 'c' :
				arg++;
				argcheck( arg, argc, argv);
				pars.cp.f = atof(  argv[arg++] );
				argcheck( arg, argc, argv);
				pars.cp.track_len = atof( argv[arg++]);
				if( pars.cp.track_len <1. ){
					fprintf(stderr,"with -c option must specify both f and track_len>0\n");
					usage();
					}
				break;
            case 'X' :
                arg++;
                pars.mfs.mdSum = atoi( argv[arg++]);
                pars.mfs.mdArr = (int *)malloc( (unsigned)( pars.mfs.mdSum*sizeof( int)) ) ;

                for(mdcc=0;mdcc<pars.mfs.mdSum;mdcc++) {
                    pars.mfs.mdArr[mdcc] = atoi( argv[arg++]) - 1; // -1 to 0-based
                    if(pars.mfs.mdArr[mdcc] < 0) {
                        fprintf(stderr,"\nThe position of missing data must larger than or equal to 0. \n");
                        exit(1);
                    }
                }
               // printf("mdSum: %d.\n", mdSum);
              //  for(mdcc=0;mdcc<mdSum;mdcc++) {
              //      printf("Each element: %d\n", pars.mfs.mdArr[mdcc]);
              //  }
                break;
            case 'Y' :
                arg++;
                pars.mfs.singlton = atoi( argv[arg++]);
                pars.mfs.doubleton = atoi( argv[arg++]);
                pars.mfs.otherton = atoi( argv[arg++]);
                break;
            case 'U' :
                arg++;
                pars.mfs.unfolded = atoi( argv[arg++]);
                break;
            case 'H' :
                arg++;
                pars.mfs.multipleHit = atoi( argv[arg++]);
                pars.mfs.mhlength = atoi( argv[arg++]);
                break;
			case 't' :
				arg++;
				argcheck( arg, argc, argv);
				pars.mp.theta = atof(  argv[arg++] );
				break;
			case 's' :
				arg++;
				argcheck( arg, argc, argv);
				if( argv[arg-1][2] == 'e' ){  /* command line seeds */
					pars.commandlineseedflag = 1 ;
					if( count == 0 ) nseeds = commandlineseed(argv+arg );
					arg += nseeds ;
				}
				else {
				    pars.mp.segsitesin = atoi(  argv[arg++] );
				}
				break;
			case 'F' :
				arg++;
				argcheck( arg, argc, argv);
				pars.mp.mfreq = atoi(  argv[arg++] );
                                if( (pars.mp.mfreq < 2 ) || (pars.mp.mfreq > pars.cp.nsam/2 ) ){
                                    fprintf(stderr," mfreq must be >= 2 and <= nsam/2.\n");
                                    usage();
                                    }
				break;
			case 'T' :
				pars.mp.treeflag = 1 ;
				arg++;
				break;
			case 'L' :
				pars.mp.timeflag = 1 ;
				arg++;
				break;
			case 'I' :
			    arg++;
			    if( count == 0 ) {
				argcheck( arg, argc, argv);
			       	pars.cp.npop = atoi( argv[arg]);
			        pars.cp.config = (int *) realloc( pars.cp.config, (unsigned)( pars.cp.npop*sizeof( int)));
				npop = pars.cp.npop ;
				}
			    arg++;
			    for( i=0; i< pars.cp.npop; i++) {
				argcheck( arg, argc, argv);
				pars.cp.config[i] = atoi( argv[arg++]);
				}
			    if( count == 0 ){
				pars.cp.mig_mat =
                                        (double **)realloc(pars.cp.mig_mat, (unsigned)(pars.cp.npop*sizeof(double *) )) ;
				pars.cp.mig_mat[0] =
                                         (double *)realloc(pars.cp.mig_mat[0], (unsigned)( pars.cp.npop*sizeof(double)));
				for(i=1; i<pars.cp.npop; i++) pars.cp.mig_mat[i] =
                                         (double *)malloc( (unsigned)( pars.cp.npop*sizeof(double)));
				pars.cp.size = (double *)realloc( pars.cp.size, (unsigned)( pars.cp.npop*sizeof( double )));
				pars.cp.alphag =
                                          (double *) realloc( pars.cp.alphag, (unsigned)( pars.cp.npop*sizeof( double )));
			        for( i=1; i< pars.cp.npop ; i++) {
				   (pars.cp.size)[i] = (pars.cp.size)[0]  ;
				   (pars.cp.alphag)[i] = (pars.cp.alphag)[0] ;
				   }
			        }
			     if( (arg <argc) && ( argv[arg][0] != '-' ) ) {
				argcheck( arg, argc, argv);
				migr = atof(  argv[arg++] );
				}
			     else migr = 0.0 ;
			     for( i=0; i<pars.cp.npop; i++)
				    for( j=0; j<pars.cp.npop; j++) pars.cp.mig_mat[i][j] = migr/(pars.cp.npop-1) ;
			     for( i=0; i< pars.cp.npop; i++) pars.cp.mig_mat[i][i] = migr ;
			     break;
			case 'm' :
			     if( npop < 2 ) { fprintf(stderr,"Must use -I option first.\n"); usage();}
			     if( argv[arg][2] == 'a' ) {
				    arg++;
				    for( pop = 0; pop <npop; pop++)
				      for( pop2 = 0; pop2 <npop; pop2++){
					     argcheck( arg, argc, argv);
					     pars.cp.mig_mat[pop][pop2]= atof( argv[arg++] ) ;
					  }
				    for( pop = 0; pop < npop; pop++) {
					  pars.cp.mig_mat[pop][pop] = 0.0 ;
					  for( pop2 = 0; pop2 < npop; pop2++){
					    if( pop2 != pop ) pars.cp.mig_mat[pop][pop] += pars.cp.mig_mat[pop][pop2] ;
					  }
				    }
				}
			    else {
		             arg++;
			         argcheck( arg, argc, argv);
		             i = atoi( argv[arg++] ) -1;
			         argcheck( arg, argc, argv);
		             j = atoi( argv[arg++] ) -1;
			         argcheck( arg, argc, argv);
		             mij = atof( argv[arg++] );
		             pars.cp.mig_mat[i][i] += mij -  pars.cp.mig_mat[i][j]  ;
		             pars.cp.mig_mat[i][j] = mij;
			    }
				break;
			case 'n' :
			     if( npop < 2 ) { fprintf(stderr,"Must use -I option first.\n"); usage();}
			    arg++;
			    argcheck( arg, argc, argv);
			    pop = atoi( argv[arg++] ) -1;
			    argcheck( arg, argc, argv);
			    psize = atof( argv[arg++] );
			    pars.cp.size[pop] = psize ;
			   break;
			case 'g' :
			     if( npop < 2 ) { fprintf(stderr,"Must use -I option first.\n"); usage();}
			    arg++;
			    argcheck( arg, argc, argv);
			    pop = atoi( argv[arg++] ) -1;
			    if( arg >= argc ) { fprintf(stderr,"Not enough arg's after -G.\n"); usage(); }
			    palpha = atof( argv[arg++] );
			    pars.cp.alphag[pop] = palpha ;
			   break;
			case 'G' :
			    arg++;
			    if( arg >= argc ) { fprintf(stderr,"Not enough arg's after -G.\n"); usage(); }
			    palpha = atof( argv[arg++] );
			    for( i=0; i<pars.cp.npop; i++)
			       pars.cp.alphag[i] = palpha ;
			   break;
			case 'e' :
			    pt = (struct devent *)malloc( sizeof( struct devent) ) ;
			    pt->detype = argv[arg][2] ;
			    ch3 = argv[arg][3] ;
			    arg++;
			    argcheck( arg, argc, argv);
			    pt->time = atof( argv[arg++] ) ;
			    pt->nextde = NULL ;
			    if( pars.cp.deventlist == NULL )
				    pars.cp.deventlist = pt ;
			    else if ( pt->time < pars.cp.deventlist->time ) {
				    ptemp = pars.cp.deventlist ;
				    pars.cp.deventlist = pt ;
				    pt->nextde = ptemp ;
				}
			    else
				   addtoelist( pt, pars.cp.deventlist ) ;
			    switch( pt->detype ) {
				case 'N' :
			          argcheck( arg, argc, argv);
				      pt->paramv = atof( argv[arg++] ) ;
				      break;
				case 'G' :
				  if( arg >= argc ) { fprintf(stderr,"Not enough arg's after -eG.\n"); usage(); }
				  pt->paramv = atof( argv[arg++] ) ;
				  break;
				case 'M' :
				    argcheck( arg, argc, argv);
				    pt->paramv = atof( argv[arg++] ) ;
				    break;
				case 'n' :
			          argcheck( arg, argc, argv);
				  pt->popi = atoi( argv[arg++] ) -1 ;
			          argcheck( arg, argc, argv);
				  pt->paramv = atof( argv[arg++] ) ;
				  break;
				case 'g' :
			          argcheck( arg, argc, argv);
				  pt->popi = atoi( argv[arg++] ) -1 ;
				  if( arg >= argc ) { fprintf(stderr,"Not enough arg's after -eg.\n"); usage(); }
				  pt->paramv = atof( argv[arg++] ) ;
				  break;
				case 's' :
			          argcheck( arg, argc, argv);
				  pt->popi = atoi( argv[arg++] ) -1 ;
			          argcheck( arg, argc, argv);
				  pt->paramv = atof( argv[arg++] ) ;
				  break;
				case 'm' :
				  if( ch3 == 'a' ) {
				     pt->detype = 'a' ;
				     argcheck( arg, argc, argv);
				     npop2 = atoi( argv[arg++] ) ;
				     pt->mat = (double **)malloc( (unsigned)npop2*sizeof( double *) ) ;
				     for( pop =0; pop <npop2; pop++){
					   (pt->mat)[pop] = (double *)malloc( (unsigned)npop2*sizeof( double) );
					   for( i=0; i<npop2; i++){
					     if( i == pop ) arg++;
					     else {
				               argcheck( arg, argc, argv);
					       (pt->mat)[pop][i] = atof( argv[arg++] ) ;
					     }
					   }
				     }
				     for( pop = 0; pop < npop2; pop++) {
					    (pt->mat)[pop][pop] = 0.0 ;
					    for( pop2 = 0; pop2 < npop2; pop2++){
					       if( pop2 != pop ) (pt->mat)[pop][pop] += (pt->mat)[pop][pop2] ;
					    }
				     }
				  }
				  else {
			            argcheck( arg, argc, argv);
				        pt->popi = atoi( argv[arg++] ) -1 ;
			            argcheck( arg, argc, argv);
				        pt->popj = atoi( argv[arg++] ) -1 ;
			            argcheck( arg, argc, argv);
				        pt->paramv = atof( argv[arg++] ) ;
				  }
				  break;
				case 'j' :
			          argcheck( arg, argc, argv);
				  pt->popi = atoi( argv[arg++] ) -1 ;
			          argcheck( arg, argc, argv);
				  pt->popj = atoi( argv[arg++] ) -1 ;
				  break;
				default: fprintf(stderr,"e event\n");  usage();
			    }
			 break;
			default: fprintf(stderr," option default\n");  usage() ;
			}
		}
        if( (pars.mp.theta == 0.0) && ( pars.mp.segsitesin == 0 ) && ( pars.mp.treeflag == 0 ) && (pars.mp.timeflag == 0) && (pars.mfs.singlton == 0) && (pars.mfs.doubleton == 0) && (pars.mfs.otherton == 0) ) {
			fprintf(stderr," either -s or -t or -Y or -T option must be used. \n");
			usage();
			exit(1);
        }
		sum = 0 ;
		for( i=0; i< pars.cp.npop; i++) sum += (pars.cp.config)[i] ;
		if( sum != pars.cp.nsam ) {
			fprintf(stderr," sum sample sizes != nsam\n");
			usage();
			exit(1);
        }
        if((pars.mp.segsitesin > 0) && (pars.mfs.unfolded != 1)){
            fprintf(stderr,"\nCannot set -s and -U simultaneously.\n");
            usage();
			exit(1);
        }

        if((pars.mfs.unfolded != 1) && (pars.cp.nsam < 6)) {
            fprintf(stderr,"\nWith folded MFS, sample size must be equal or greater than 6. \n");
            usage();
			exit(1);
        }

        if((pars.mp.segsitesin > 0) && ((pars.mfs.singlton > 0) || (pars.mfs.doubleton > 0) || (pars.mfs.otherton > 0))){
            fprintf(stderr,"\nCannot set -s and -Y simultaneously.\n");
            usage();
			exit(1);
        }

        if((pars.mfs.mdSum > 0) && (pars.mfs.mhlength > 0)) {
            fprintf(stderr,"\nCannot set -X and -H simultaneously.\n");
            usage();
			exit(1);
        }

        if(((pars.mfs.mdSum > 0) && ( pars.mp.theta > 0)) || ((pars.mfs.mdSum > 0) && ( pars.mp.segsitesin > 0)))  {
            fprintf(stderr,"\nCannot set -X and -t or -s simultaneously.\n");
            usage();
			exit(1);
        }

        if((pars.mfs.mdSum > 0) && (pars.mfs.unfolded == 1)) {
            fprintf(stderr,"\nCannot set -X and -U 1 simultaneously.\n");
            usage();
			exit(1);
        }

}


	void
argcheck( int arg, int argc, char *argv[] )
{
	if( (arg >= argc ) || ( argv[arg][0] == '-') ) {
	   fprintf(stderr,"not enough arguments after %s\n", argv[arg-1] ) ;
	   fprintf(stderr,"For usage type: ms<return>\n");
	   exit(0);
	  }
}

	void
usage()
{
fprintf(stderr,"usage: ms nsam howmany \n");
fprintf(stderr,"  Options: \n");
fprintf(stderr,"\t -t theta   (this option and/or the next must be used. Theta = 4*N0*u )\n");
fprintf(stderr,"\t -s segsites   ( fixed number of segregating sites)\n");
fprintf(stderr,"\t -Y singleton doubleton tripleton  (fixed number of MFS)\n");
fprintf(stderr,"\t -U 1/0 (1 represents unfolded MFS; 0 represents folded MFS. Default is 1.)\n");
fprintf(stderr,"\t -H 0/1 mhlength (0 represents no multiple hit; 1 represents multiple hit. Default is 0. If you specify 1, you must specify 'mhlength'.)\n");
fprintf(stderr,"\t -X quesMarkNum qmOnepos qmTwopos qmThreepos ...These positions are column-based. The number of first column is 1,2,3...nsam. The number of second column is nsam+1, nsam+2, 2*nsam. We recommend to use the which function in R to obtain question marks index (i.e., which(yourData == '?'))\n");
fprintf(stderr,"\t -T          (Output gene tree.)\n");
fprintf(stderr,"\t -F minfreq     Output only sites with freq of minor allele >= minfreq.\n");
fprintf(stderr,"\t -r rho nsites     (rho here is 4Nc)\n");
fprintf(stderr,"\t\t -c f track_len   (f = ratio of conversion rate to rec rate. tracklen is mean length.) \n");
fprintf(stderr,"\t\t\t if rho = 0.,  f = 4*N0*g, with g the gene conversion rate.\n");
fprintf(stderr,"\t -G alpha  ( N(t) = N0*exp(-alpha*t) .  alpha = -log(Np/Nr)/t\n");
fprintf(stderr,"\t -I npop n1 n2 ... [mig_rate] (all elements of mig matrix set to mig_rate/(npop-1) \n");
fprintf(stderr,"\t\t -m i j m_ij    (i,j-th element of mig matrix set to m_ij.)\n");
fprintf(stderr,"\t\t -ma m_11 m_12 m_13 m_21 m_22 m_23 ...(Assign values to elements of migration matrix.)\n");
fprintf(stderr,"\t\t -n i size_i   (popi has size set to size_i*N0 \n");
fprintf(stderr,"\t\t -g i alpha_i  (If used must appear after -M option.)\n");
fprintf(stderr,"\t   The following options modify parameters at the time 't' specified as the first argument:\n");
fprintf(stderr,"\t -eG t alpha  (Modify growth rate of all pop's.)\n");
fprintf(stderr,"\t -eg t i alpha_i  (Modify growth rate of pop i.) \n");
fprintf(stderr,"\t -eM t mig_rate   (Modify the mig matrix so all elements are mig_rate/(npop-1)\n");
fprintf(stderr,"\t -em t i j m_ij    (i,j-th element of mig matrix set to m_ij at time t )\n");
fprintf(stderr,"\t -ema t npop  m_11 m_12 m_13 m_21 m_22 m_23 ...(Assign values to elements of migration matrix.)\n");
fprintf(stderr,"\t -eN t size  (Modify pop sizes. New sizes = size*N0 ) \n");
fprintf(stderr,"\t -en t i size_i  (Modify pop size of pop i.  New size of popi = size_i*N0 .)\n");
fprintf(stderr,"\t -es t i proportion  (Split: pop i -> pop-i + pop-npop, npop increases by 1.\n");
fprintf(stderr,"\t\t proportion is probability that each lineage stays in pop-i. (p, 1-p are admixt. proport.\n");
fprintf(stderr,"\t\t Size of pop npop is set to N0 and alpha = 0.0 , size and alpha of pop i are unchanged.\n");
fprintf(stderr,"\t -ej t i j   ( Join lineages in pop i and pop j into pop j\n");
fprintf(stderr,"\t\t  size, alpha and M are unchanged.\n");
fprintf(stderr,"\t  -f filename     ( Read command line arguments from file filename.)\n");
fprintf(stderr,"\t  -p n ( Specifies the precision of the position output.  n is the number of digits after the decimal.)\n");
fprintf(stderr," See msdoc.pdf for explanation of these parameters.\n");

exit(1);
}
	void
addtoelist( struct devent *pt, struct devent *elist )
{
	struct devent *plast, *pevent, *ptemp  ;

	pevent = elist ;
	while(  (pevent != NULL ) && ( pevent->time <= pt->time ) )  {
		plast = pevent ;
		pevent = pevent->nextde ;
		}
	ptemp = plast->nextde ;
	plast->nextde = pt ;
	pt->nextde = ptemp ;
}

	void
free_eventlist( struct devent *pt, int npop )
{
   struct devent *next ;
   int pop ;

   while( pt != NULL){
	  next = pt->nextde ;
	  if( pt->detype == 'a' ) {
	     for( pop = 0; pop < npop; pop++) free( (pt->mat)[pop] );
		 free( pt->mat );
	  }
	  free(pt);
	  pt = next ;
   }
}


/************ make_gametes.c  *******************************************
*
*
*****************************************************************************/

#define STATE0 '0'
#define STATE1 '1'
#define STATE2 '2'
#define STAR   '-'
#define QUMA   '?'

	int
make_gametes(int nsam, int mfreq, double *posit, long int nsites, struct node *ptree, double tt, int newsites, int ns, char **list )
{
	int  tip, j,  node, lastNode;
        int pickb(int nsam, struct node *ptree, double tt),
            pickbmf(int nsam, int mfreq, struct node *ptree, double tt) ;
    double ran1(), randN;
    lastNode = -1;

	for(  j=ns; j< ns+newsites ;  j++ ) {
		if( mfreq == 1 ) node = pickb(  nsam, ptree, tt);
		else node = pickbmf(  nsam, mfreq, ptree, tt);

		if(pars.mfs.multipleHit == 1) { //have multiple hit
            if( j>ns ) {
                if(lastNode == -1) {
                    for( tip=0; tip < nsam ; tip++) {
                        if( tdesn(ptree, tip, node) ) list[tip][j] = STATE1 ;
                        else list[tip][j] = STATE0 ;
                    }
                    lastNode = node;
                } else {
                    if(floor(posit[j]*nsites) == floor(posit[j-1]*nsites)) {
                        if(lastNode == node) {
                        } else if(tdesn(ptree, lastNode, node) || tdesn(ptree, node, lastNode)) {
                            for( tip=0; tip < nsam ; tip++) {
                                if( tdesn(ptree, tip, lastNode) && tdesn(ptree, tip, node)) {
                                    list[tip][j-1] = STATE2;
                                } else if(tdesn(ptree, tip, lastNode) || tdesn(ptree, tip, node)) {
                                    list[tip][j-1] = STATE1;
                                } else {
                                    list[tip][j-1] = STATE0;
                                }
                            }
                        } else {
                            randN = ran1();
                            for( tip=0; tip < nsam ; tip++) {
                                if( tdesn(ptree, tip, node) ) {
                                     if(randN<1.0/3.0) {
                                        list[tip][j-1] = STATE1;
                                     } else {
                                        list[tip][j-1] = STATE2;
                                     }
                                }
                            }
                        }
                        for( tip=0; tip < nsam ; tip++) {
                            list[tip][j] = STAR;
                        }
                        lastNode = -1;
                    } else {
                        for( tip=0; tip < nsam ; tip++) {
                            if( tdesn(ptree, tip, node) ) list[tip][j] = STATE1 ;
                            else list[tip][j] = STATE0 ;
                        }
                        lastNode = node;
                    }
                }
            } else {
                for( tip=0; tip < nsam ; tip++) {
                    if( tdesn(ptree, tip, node) ) list[tip][j] = STATE1 ;
                    else list[tip][j] = STATE0 ;
                }
                lastNode = node;
            }
		} else {
            for( tip=0; tip < nsam ; tip++) {
               if( tdesn(ptree, tip, node) ) list[tip][j] = STATE1 ;
               else list[tip][j] = STATE0 ;
            }
        }
    }
}


int
make_MFSgametes(int nsam, int mfreq, double *posit, long int mhlen, struct node *ptree, double sintontt, double doutontt, double othtontt, int stonnewsites, int dtonnewsites, int otonnewsites, int ns, char **list, int loop)
{
	int  tip, j, i, node, ttsites, randomN, lastNode;
	double y, ran1(), randN;
    int pickb_Ston(int nsam, struct node *ptree, double tt),  pickbmf_Ston(int nsam, int mfreq, struct node *ptree, double tt) ;
    int pickb_Dton(int nsam, struct node *ptree, double tt),  pickbmf_Dton(int nsam, int mfreq, struct node *ptree, double tt) ;
    int pickb_Oton(int nsam, struct node *ptree, double tt),  pickbmf_Oton(int nsam, int mfreq, struct node *ptree, double tt) ;

    lastNode = -1;

    j = ns;
    ttsites = stonnewsites+dtonnewsites+otonnewsites;
    while(ttsites>0) {
        randomN = (int)(1+ran1()*ttsites);
        if(randomN <= stonnewsites) {
			stonnewsites--;
			if(mfreq == 1) {
                node = pickb_Ston(nsam, ptree, sintontt);
			} else node = pickbmf_Ston(nsam, mfreq, ptree, sintontt);
        } else if(randomN <= stonnewsites+dtonnewsites) {
			dtonnewsites--;
			if(mfreq == 1) {
                node = pickb_Dton(nsam, ptree, doutontt);
			} else node = pickbmf_Dton(  nsam, mfreq, ptree, doutontt);
		} else {
			otonnewsites--;
			if(mfreq == 1) {
                node = pickb_Oton(nsam, ptree, othtontt);
			} else node = pickbmf_Oton(  nsam, mfreq, ptree, othtontt);
		}

        if(pars.mfs.multipleHit == 1) { //have multiple hit
            if( j>ns ) {
                if(lastNode == -1) {
                    for( tip=0; tip < nsam ; tip++) {
                        if( tdesn(ptree, tip, node) ) list[tip+loop*nsam][j] = STATE1 ;
                        else list[tip+loop*nsam][j] = STATE0 ;
                    }
                    lastNode = node;
                } else {
                    if(floor(posit[j]*mhlen) == floor(posit[j-1]*mhlen)) {
                        if(lastNode == node) {
                        } else if(tdesn(ptree, lastNode, node) || tdesn(ptree, node, lastNode)) {
                            for( tip=0; tip < nsam ; tip++) {
                                if( tdesn(ptree, tip, lastNode) && tdesn(ptree, tip, node)) {
                                    list[tip+loop*nsam][j-1] = STATE2;
                                } else if(tdesn(ptree, tip, lastNode) || tdesn(ptree, tip, node)) {
                                    list[tip+loop*nsam][j-1] = STATE1;
                                } else {
                                    list[tip+loop*nsam][j-1] = STATE0;
                                }
                            }
                        } else {
                            randN = ran1();
                            for( tip=0; tip < nsam ; tip++) {
                                if( tdesn(ptree, tip, node) ) {
                                     if(randN<1.0/3.0) {
                                        list[tip+loop*nsam][j-1] = STATE1;
                                     } else {
                                        list[tip+loop*nsam][j-1] = STATE2;
                                     }
                                }
                            }
                        }
                        for( tip=0; tip < nsam ; tip++) {
                            list[tip+loop*nsam][j] = STAR;
                        }
                        lastNode = -1;
                    } else {
                        for( tip=0; tip < nsam ; tip++) {
                            if( tdesn(ptree, tip, node) ) list[tip+loop*nsam][j] = STATE1 ;
                            else list[tip+loop*nsam][j] = STATE0 ;
                        }
                        lastNode = node;
                    }
                }

            } else {
                for( tip=0; tip < nsam ; tip++) {
                    if( tdesn(ptree, tip, node) ) list[tip+loop*nsam][j] = STATE1 ;
                    else list[tip+loop*nsam][j] = STATE0 ;
                }

                lastNode = node;
            }
        } else { //not have multiple hit
            for(tip=0; tip < nsam ; tip++) {
                if(tdesn(ptree, tip, node) ) list[tip+loop*nsam][j] = STATE1;
                else list[tip+loop*nsam][j] = STATE0;
            }
        }

        j++;
        ttsites = stonnewsites+dtonnewsites+otonnewsites;
    }
}

int
make_MFSgametesQM(int nsam, double *posit, long int mhlen, struct node *ptree, double* sdoTemTime, int* sdoTemSite, int ns, char **list, int* currTreRow, int* currTreCol, int currTreLen)
{
	int  tip, j, i, node, ttsites, randomN, lastNode;
	double y, ran1(), randN;
    int pickb_StonQM(int nsam, struct node *ptree, double tt);
    int pickb_DtonQM(int nsam, struct node *ptree, double tt);
    int pickb_OtonQM(int nsam, struct node *ptree, double tt);
    double sintontt, doutontt, othtontt;
    int stonnewsites, dtonnewsites, otonnewsites;
    int currSiteNsam = 0;
    int k=0, cc=0, ccnn=0;
    double st = 0.0, dt = 0.0, ot = 0.0;
    int nc = 0, ancest = 0;

    sintontt = sdoTemTime[0];
    doutontt = sdoTemTime[1];
    othtontt = sdoTemTime[2];

    stonnewsites = sdoTemSite[0];
    dtonnewsites = sdoTemSite[1];
    otonnewsites = sdoTemSite[2];

    lastNode = -1;

    j = ns;
    ttsites = stonnewsites+dtonnewsites+otonnewsites;
    while(ttsites>0) {
        st = 0.0;
        dt = 0.0;
        ot = 0.0;

        currSiteNsam = 0;
        for(i=0;i<currTreLen;i++) {
            if(currTreCol[i] == j) {
                currSiteNsam++;
            }
        }

        if(currSiteNsam > 0) { // there are some question marks for this site
            int currSams[currSiteNsam];

            currSiteNsam = 0;
            for(i=0;i<currTreLen;i++) {
                if(currTreCol[i] == j) {
                    currSams[currSiteNsam++] = currTreRow[i];
                }
            }

            //update ncs with question marks
            for( i=0; i<2*nsam-2; i++) (ptree+i)->ncs = 0;
            for( i=0; i<2*nsam-2; i++) (ptree+i)->masked = 0; // very important


            for( tip=0; tip < nsam; tip++) {
                for(i=0;i<currSiteNsam;i++) {
                    if(currSams[i] == tip) {
                        (ptree+tip)->masked = 1; // the leaf is masked
                      //  (ptree+(ptree+tip)->abv)->masked = 1; // the direct parent of masked leaf is masked
                        break;
                    }
                }

                if(!((ptree+tip)->masked)) { // update ncs
                    k = (ptree+tip)->abv;
                    while(k < 2*nsam-2) {
                        cc = (ptree+k) -> ncs;
                        (ptree+k) -> ncs = cc+1;
                        k = (ptree+k)->abv;
                    }
                }
            }

	// for( tip=0; tip < 2*nsam-1 ; tip++) {
   // printf("node: %d, parent: %d, time: %f, nc: %d, masked: %d\n", tip, (ptree+tip)->abv, (ptree + (ptree+tip)->abv )->time - (ptree+tip)->time, (ptree+tip)->ncs, (ptree+tip)->masked);
 //  }
            for( i=0; i<2*nsam-2; i++) {
                if((ptree+i)->masked == 1) {
                    continue;
                }

                nc = (ptree+i)->ncs;

               // if(pars.mfs.unfolded != 1) { // folded
                    if(nc == 0) {
                        st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // singleton
                        (ptree+i)->masked = 1;
                        ancest = (ptree+i)->abv;
                        while(ancest < 2*nsam-2) {
                            ccnn = (ptree+ancest) -> ncs;
                            if(ccnn > 1) {
                                break;
                            }
                            if((ccnn == 1) && ((ptree+ancest)->masked == 0)) { //if(ccnn == 1) {
                                st += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time; // singleton
                                (ptree+ancest)->masked = 1;
                            }
                            ancest = (ptree+ancest)->abv;
                        }
                    } else if(nc == (nsam-1)) {
                        st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // singleton
                        (ptree+i)->masked = 1;
                        ancest = (ptree+i)->abv;
                        while(ancest < 2*nsam-2) {
                            ccnn = (ptree+ancest) -> ncs;
                            if(ccnn > (nsam-1)) {
                                break;
                            }
                            if((ccnn == (nsam-1)) && ((ptree+ancest)->masked == 0)) { // if(ccnn == (nsam-1)) {
                                st += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time; // singleton
                                (ptree+ancest)->masked = 1;
                            }
                            ancest = (ptree+ancest)->abv;
                        }
                    } else if(nc == 2) {
                        dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // doubleton
                        (ptree+i)->masked = 1;
                        ancest = (ptree+i)->abv;
                        while(ancest < 2*nsam-2) {
                            ccnn = (ptree+ancest) -> ncs;
                            if(ccnn > 2) {
                                break;
                            }
                            if((ccnn == 2) && ((ptree+ancest)->masked == 0)) {   // if(ccnn == 2) {
                                dt += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time; // doubleton
                                (ptree+ancest)->masked = 1;
                            }
                            ancest = (ptree+ancest)->abv;
                        }
                    } else if(nc == (nsam-2)) {
                        dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // doubleton
                        (ptree+i)->masked = 1;
                        ancest = (ptree+i)->abv;
                        while(ancest < 2*nsam-2) {
                            ccnn = (ptree+ancest) -> ncs;
                            if(ccnn > (nsam-2)) {
                                break;
                            }
                            if((ccnn == (nsam-2)) && ((ptree+ancest)->masked == 0)) {  //if(ccnn == (nsam-2)) {
                                dt += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time; // doubleton
                                (ptree+ancest)->masked = 1;
                            }
                            ancest = (ptree+ancest)->abv;
                        }
                    } else {
                        ot += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // xton
                        (ptree+i)->masked = 1;
                        ancest = (ptree+i)->abv;
                        while(ancest < 2*nsam-2) {
                            ccnn = (ptree+ancest) -> ncs;
                            if((ccnn == (nsam-1)) || (ccnn == (nsam-2))) {
                                break;
                            }
                            if((ccnn >= 3) && (ccnn != (nsam-1)) && (ccnn != (nsam-2)) && ((ptree+ancest)->masked == 0)) {  // if((ccnn >= 3) && (ccnn != (nsam-1)) && (ccnn != (nsam-2))) {
                                ot += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time; // xton
                                (ptree+ancest)->masked = 1;
                            }
                            ancest = (ptree+ancest)->abv;
                        }
                    }
                //}
            }

            for( i=0; i<2*nsam-2; i++) (ptree+i)->masked = 0; // very important
            for( tip=0; tip < nsam; tip++) {
                for(i=0;i<currSiteNsam;i++) {
                    if(currSams[i] == tip) {
                        (ptree+tip)->masked = 1; // the leaf is masked
                        //printf("tip: %d is masked \n", tip);

                      //  (ptree+(ptree+tip)->abv)->masked = 1; // the direct parent of masked leaf is masked
                        break;
                    }
                }
            }

            randomN = (int)(1+ran1()*ttsites);
            if(randomN <= stonnewsites) {
                stonnewsites--;
                node = pickb_StonQM(nsam, ptree, st);
            } else if(randomN <= stonnewsites+dtonnewsites) {
                dtonnewsites--;
                node = pickb_DtonQM(nsam, ptree, dt);
            } else {
                otonnewsites--;
                node = pickb_OtonQM(nsam, ptree, ot);
            }

            for( i=0; i<2*nsam-2; i++) (ptree+i)->masked = 0; // very important
            for( tip=0; tip < nsam; tip++) {
                for(i=0;i<currSiteNsam;i++) {
                    if(currSams[i] == tip) {
                        (ptree+tip)->masked = 1; // the leaf is masked
                                              //  (ptree+(ptree+tip)->abv)->masked = 1; // the direct parent of masked leaf is masked
                        break;
                    }
                }
            }

            for(tip=0; tip < nsam ; tip++) {
                if((ptree+tip)->masked == 1) {
                    list[tip][j] = QUMA;
                } else if( tdesn(ptree, tip, node) ) {
                    list[tip][j] = STATE1 ;
                } else {
                    list[tip][j] = STATE0 ;
                }
            }


        } else {
            //update ncs with question marks
            for( i=0; i<2*nsam-2; i++) (ptree+i)->ncs = 0;
            for( i=0; i<2*nsam-2; i++) (ptree+i)->masked = 0; // very important

            for( tip=0; tip < nsam; tip++) {
                k = (ptree+tip)->abv;
                while(k < 2*nsam-2) {
                    cc = (ptree+k) -> ncs;
                    (ptree+k) -> ncs = cc+1;
                    k = (ptree+k)->abv;
                }
            }

            st = sintontt;
            dt = doutontt;
            ot = othtontt;

            randomN = (int)(1+ran1()*ttsites);
            if(randomN <= stonnewsites) {
                stonnewsites--;
                node = pickb_Ston(nsam, ptree, st);
            } else if(randomN <= stonnewsites+dtonnewsites) {
                dtonnewsites--;
                node = pickb_Dton(nsam, ptree, dt);
            } else {
                otonnewsites--;
                node = pickb_Oton(nsam, ptree, ot);
            }


            for(tip=0; tip < nsam ; tip++) {
                if((ptree+tip)->masked == 1) {
                    list[tip][j] = QUMA;
                } else if( tdesn(ptree, tip, node) ) {
                    list[tip][j] = STATE1 ;
                } else {
                    list[tip][j] = STATE0 ;
                }
            }
        }

//	 for( tip=0; tip < 2*nsam-1 ; tip++) {
 //   printf("node: %d, parent: %d, time: %f, nc: %d, masked: %d\n", tip, (ptree+tip)->abv, (ptree + (ptree+tip)->abv )->time - (ptree+tip)->time, (ptree+tip)->ncs, (ptree+tip)->masked);
 //  }


       /* if(pars.mfs.multipleHit == 1) { //have multiple hit
            if( j>ns ) {
                if(lastNode == -1) {
                    for( tip=0; tip < nsam ; tip++) {
                        if((ptree+tip)->masked == 1) {
                            list[tip][j] = QUMA;
                        } else if( tdesn(ptree, tip, node) ) {
                            list[tip][j] = STATE1 ;
                        } else {
                            list[tip][j] = STATE0 ;
                        }
                    }
                    lastNode = node;
                } else {
                    if(floor(posit[j]*mhlen) == floor(posit[j-1]*mhlen)) {
                        if(lastNode == node) {
                        } else if(tdesn(ptree, lastNode, node) || tdesn(ptree, node, lastNode)) {
                            for( tip=0; tip < nsam ; tip++) {
                                if((ptree+tip)->masked == 1) {
                                    list[tip][j-1] = QUMA;
                                } else if( tdesn(ptree, tip, lastNode) && tdesn(ptree, tip, node)) {
                                    list[tip][j-1] = STATE2;
                                } else if(tdesn(ptree, tip, lastNode) || tdesn(ptree, tip, node)) {
                                    list[tip][j-1] = STATE1;
                                } else {
                                    list[tip][j-1] = STATE0;
                                }
                            }
                        } else {
                            randN = ran1();
                            for( tip=0; tip < nsam ; tip++) {
                                if( tdesn(ptree, tip, node) ) {
                                     if((ptree+tip)->masked == 1) {
                                        list[tip][j-1] = QUMA;
                                     } else if(randN<1.0/3.0) {
                                        list[tip][j-1] = STATE1;
                                     } else {
                                        list[tip][j-1] = STATE2;
                                     }
                                }
                            }
                        }
                        for( tip=0; tip < nsam ; tip++) {
                            list[tip][j] = STAR;
                        }
                        lastNode = -1;
                    } else {
                        for( tip=0; tip < nsam ; tip++) {
                            if((ptree+tip)->masked == 1) {
                                list[tip][j] = QUMA;
                            } else if( tdesn(ptree, tip, node) ) {
                                list[tip][j] = STATE1 ;
                            } else {
                                list[tip][j] = STATE0 ;
                            }
                        }
                        lastNode = node;
                    }
                }

            } else {
                for( tip=0; tip < nsam ; tip++) {
                    if((ptree+tip)->masked == 1) {
                        list[tip][j] = QUMA;
                    } else if( tdesn(ptree, tip, node) ) {
                        list[tip][j] = STATE1 ;
                    } else {
                        list[tip][j] = STATE0 ;
                    }
                }

                lastNode = node;
            }
        } else { //not have multiple hit */
        // when have question marks, there are not multiple hit

        	// for( tip=0; tip < 2*nsam-1 ; tip++) {
  //  printf("node: %d, parent: %d, time: %f, nc: %d, masked: %d\n", tip, (ptree+tip)->abv, (ptree + (ptree+tip)->abv )->time - (ptree+tip)->time, (ptree+tip)->ncs, (ptree+tip)->masked);
 //  }



        j++;
        ttsites = stonnewsites+dtonnewsites+otonnewsites;
    }
}

/***  ttime.c : Returns the total time in the tree, *ptree, with nsam tips. **/

	double
ttime( ptree, nsam)
	struct node *ptree;
	int nsam;
{
	double t;
	int i;

	t = (ptree + 2*nsam-2) -> time ;
	for( i=nsam; i< 2*nsam-1 ; i++)
		t += (ptree + i)-> time ;
	return(t);
}


	double
ttimemf( ptree, nsam, mfreq)
	struct node *ptree;
	int nsam, mfreq;
{
	double t;
	int i;

	t = 0. ;
	for( i=0;  i< 2*nsam-2  ; i++)
	  if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) )
		t += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
	return(t);
}

/***  sdottime: Returns the total time of singleton, doubleton, xton in the tree **/
void sdottime(struct node *ptree, int nsam, double* dotime, int unfo)
{
    double st, dt, ot;
	int i, tip, cc, k, nc;
	st = 0.0;
	dt = 0.0;
	ot = 0.0;

    // init number of children
    for( i=0; i<2*nsam-2; i++) (ptree+i)->ncs = 0;
    for( tip=0; tip < nsam; tip++) {
        k = (ptree+tip)->abv;
        while(k < 2*nsam-2) {
            cc = (ptree+k) -> ncs;
            (ptree+k) -> ncs = cc+1;
            k = (ptree+k)->abv;
        }
    }


	for( i=0; i<2*nsam-2; i++) {
        nc = (ptree+i)->ncs;

        if(unfo != 1) { // folded
            if((nc == 0) || (nc == (nsam-1)))  {
                st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // singleton
            } else if ((nc == 2) || (nc == (nsam-2))) {
                dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // doubleton
            } else {
                ot += (ptree + (ptree+i)->abv )->time - (ptree+i)->time; // xton
            }

        } else { // unfolded
            if(nc == 0) {
                st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
            } else if (nc == 2) {
                dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
            } else {
                ot += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
            }
        }
    }

    *(dotime) = st;
    *(dotime+1) = dt;
    *(dotime+2) = ot;
}

/***  sdottimemf: With mfreq, Returns the total time of singleton, doubleton, xton in the tree **/
void sdottimemf(struct node *ptree, int nsam, double* dotime, int mfreq, int unfo)
{
    double st, dt, ot;
	int i, tip, cc, k, nc;
	st = 0.0;
	dt = 0.0;
	ot = 0.0;


    // init number of children
    for( i=0; i<2*nsam-2; i++) (ptree+i)->ncs = 0;
    for( tip=0; tip < nsam; tip++) {
        k = (ptree+tip)->abv;
        while(k < 2*nsam-2) {
            cc = (ptree+k) -> ncs;
            (ptree+k) -> ncs = cc+1;
            k = (ptree+k)->abv;
        }
    }


    for( i=0; i<2*nsam-2; i++) {
        nc = (ptree+i)->ncs;

        if(unfo != 1) { // folded
            if((nc == 0) || (nc == (nsam-1)))  {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            } else if ((nc == 2) || (nc == (nsam-2))) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            } else {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    ot += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            }

        } else { // unfolded
            if(nc == 0) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    st += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            } else if (nc == 2) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    dt += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            } else {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    ot += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
                }
            }
        }
    }

    *(dotime) = st;
    *(dotime+1) = dt;
    *(dotime+2) = ot;


}

	void
prtree( ptree, nsam)
	struct node *ptree;
	int nsam;
{
	double t;
	int i, *descl, *descr ;
	void parens( struct node *ptree, int *descl, int *descr, int noden );

	descl = (int *)malloc( (unsigned)(2*nsam-1)*sizeof( int) );
	descr = (int *)malloc( (unsigned)(2*nsam-1)*sizeof( int) );
	for( i=0; i<2*nsam-1; i++) descl[i] = descr[i] = -1 ;
	for( i = 0; i< 2*nsam-2; i++){
	  if( descl[ (ptree+i)->abv ] == -1 ) descl[(ptree+i)->abv] = i ;
	  else descr[ (ptree+i)->abv] = i ;
	 }
	parens( ptree, descl, descr, 2*nsam-2);
	free( descl ) ;
	free( descr ) ;
}

	void
parens( struct node *ptree, int *descl, int *descr,  int noden)
{
	double time ;

   if( descl[noden] == -1 ) {
	printf("%d:%5.3lf", noden+1, (ptree+ ((ptree+noden)->abv))->time );
	}
   else{
	printf("(");
	parens( ptree, descl,descr, descl[noden] ) ;
	printf(",");
	parens(ptree, descl, descr, descr[noden] ) ;
	if( (ptree+noden)->abv == 0 ) printf(");\n");
	else {
	  time = (ptree + (ptree+noden)->abv )->time - (ptree+noden)->time ;
	  printf("):%5.3lf", time );
	  }
        }
}

/***  pickb : returns a random branch from the tree. The probability of picking
              a particular branch is proportional to its duration. tt is total
	      time in tree.   ****/

	int
pickb(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
	double x, y, ran1();
	int i;

	x = ran1()*tt;
	for( i=0, y=0; i < 2*nsam-2 ; i++) {
		y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
		if( y >= x ) return( i ) ;
		}
	return( 2*nsam - 3  );  /* changed 4 Feb 2010 */
}

	int
pickbmf(nsam, mfreq, ptree, tt )
	int nsam, mfreq;
	struct node *ptree;
	double tt;
{
	double x, y, ran1();
	int i, lastbranch = 0 ;

	x = ran1()*tt;
	for( i=0, y=0; i < 2*nsam-2 ; i++) {
	  if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ){
		y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time ;
		lastbranch = i ;    /* changed 4 Feb 2010 */
	  }
	  if( y >= x ) return( i ) ;
	}
	return( lastbranch );   /*  changed 4 Feb 2010 */
}

int
pickb_Ston(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
	double x, y=0.0, ran1();
	int i, nc;
    x = ran1()*tt;
	if(pars.mfs.unfolded != 1) {
        for( i=0, y=0; i < 2*nsam-2 ; i++) {
            nc = (ptree+i)->ncs;
            if((nc == 0) || (nc == (nsam-1))) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    } else {
        for( i=0, y=0; i < nsam; i++) {
            y += (ptree + (ptree+i)->abv )->time;
            if( y >= x ) return( i ) ;
        }
    }
	return( nsam-1 );
}


	int
pickb_Dton(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
    double x, y=0.0, ran1();
	int i, j, nc;
	int tip;
    x = ran1()*tt;

    if(pars.mfs.unfolded != 1) {
        for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            nc = (ptree+i)->ncs;
            if((nc == 2) || (nc == (nsam-2))) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    } else {
    	for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            if((ptree+i)->ncs == 2) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    }
	//printf("Warning: Have selected the LastNode: %d\n", lastNode);
	//return( lastNode );
}

int
pickb_Oton(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
    double x, y=0.0, ran1();
	int i, j, nc;
	int tip;
    x = ran1()*tt;

    if(pars.mfs.unfolded != 1) {
         for( i=nsam, y=0; i < 2*nsam-2; i++) {
            nc = (ptree+i)->ncs;
            if((nc >= 3) && (nc != (nsam-1)) && (nc != (nsam-2))) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    } else {
        for( i=nsam, y=0; i < 2*nsam-2; i++) {
            if((ptree+i)->ncs >=3 ) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    }


	//	printf("Warning: Have selected the node : %d\n", 2*nsam-3);
	return( 2*nsam-3 );
}

int
pickbmf_Oton(nsam, mfreq, ptree, tt )
	int nsam, mfreq;
	struct node *ptree;
	double tt;
{
    double x, y, ran1();
	int i, lastbranch = 0 ;
	int tip;
    x = ran1()*tt;

    if(pars.mfs.unfolded != 1) {
         for( i=nsam, y=0; i < 2*nsam-2; i++) {
            if(((ptree+i)->ncs >=3) && ((ptree+i)->ncs != (nsam-1)) && ((ptree+i)->ncs != (nsam-2))) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ){
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    lastbranch = i ;
                }
                if( y >= x ) return( i ) ;
            }
        }
    } else {
        for( i=nsam, y=0; i < 2*nsam-2; i++) {
            if((ptree+i)->ncs >=3 ) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ){
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    lastbranch = i ;
                }
                if( y >= x ) return( i ) ;
            }
        }
    }
	return( lastbranch );
}

	int
pickbmf_Dton(nsam, mfreq, ptree, tt )
	int nsam, mfreq;
	struct node *ptree;
	double tt;
{
    double x, y, ran1();
	int i, lastbranch = 0 ;

	int tip;
    x = ran1()*tt;

    if(pars.mfs.unfolded != 1) {
        for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            if(((ptree+i)->ncs == 2) || ((ptree+i)->ncs == (nsam-2))) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ){
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    lastbranch = i ;
                }
                if( y >= x ) return( i ) ;
            }
        }
    } else {
    	for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            if((ptree+i)->ncs == 2) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ){
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    lastbranch = i ;
                }
                if( y >= x ) return( i ) ;
            }
        }
    }
	//printf("Warning: Have selected the LastNode: %d\n", lastNode);
	return( lastbranch );
}


int
pickbmf_Ston(nsam, mfreq, ptree, tt )
	int nsam, mfreq;
	struct node *ptree;
	double tt;
{
	double x, y, ran1();
	int i, lastbranch = 0 ;
    x = ran1()*tt;
	if(pars.mfs.unfolded != 1) {
        for( i=0, y=0; i < 2*nsam-2 ; i++) {
            if(((ptree+i)->ncs == 0) || ((ptree+i)->ncs == (nsam-1))) {
                if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    lastbranch = i ;
                }
                if( y >= x ) return( i ) ;
            }
        }
    } else {
        for( i=0, y=0; i < nsam; i++) {
            if( ( (ptree+i)->ndes >= mfreq )  && ( (ptree+i)->ndes <= nsam-mfreq) ) {
                y += (ptree + (ptree+i)->abv )->time;
                lastbranch = i ;
            }
            if( y >= x ) return( i ) ;
        }
    }
	return(lastbranch);
}


int
pickb_StonQM(int nsam, struct node *ptree, double tt)
{
	double x, y=0.0, ran1();
	int i, nc, ancest, ccnn;
    x = ran1()*tt;
	//if(pars.mfs.unfolded != 1) {
        for( i=0, y=0; i < nsam; i++) {
            if((ptree+i)->masked == 1) {
                continue;
            }

            nc = (ptree+i)->ncs;

            if(nc == 0) {

                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                (ptree+i)->masked = 1;
                if( y >= x ) return( i ) ;

                ancest = (ptree+i)->abv;
                while(ancest < 2*nsam-2) {
                    ccnn = (ptree+ancest) -> ncs;
                    if(ccnn > 1) {
                        break;
                    }
                    if((ccnn == 1) && ((ptree+ancest)->masked == 0)) {
                        y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                        (ptree+ancest)->masked = 1;

                        if( y >= x ) return( ancest ) ;
                    }
                    ancest = (ptree+ancest)->abv;
                }

            } else if(nc == (nsam-1)) {

                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                (ptree+i)->masked = 1;
                if( y >= x ) return( i ) ;

                ancest = (ptree+i)->abv;
                while(ancest < 2*nsam-2) {
                    ccnn = (ptree+ancest) -> ncs;
                    if(ccnn > (nsam-1)) {
                        break;
                    }
                    if((ccnn == (nsam-1)) && ((ptree+ancest)->masked == 0)) {
                        y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                        (ptree+ancest)->masked = 1;

                        if( y >= x ) return( ancest ) ;
                    }
                    ancest = (ptree+ancest)->abv;
                }

            }

        }
    /*} else {
        for( i=0, y=0; i < nsam; i++) {
            y += (ptree + (ptree+i)->abv )->time;
            if( y >= x ) return( i ) ;
        }
    }*/
	return( nsam-1 );
}


int
pickb_DtonQM(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
    double x, y=0.0, ran1();
	int i, j, nc, ccnn;
	int tip, ancest;
    x = ran1()*tt;

  //  if(pars.mfs.unfolded != 1) {
        for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            if((ptree+i)->masked == 1) {
                continue;
            }

            nc = (ptree+i)->ncs;

            if(nc == 2) {

                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                (ptree+i)->masked = 1;

                if( y >= x ) return( i ) ;
                // only start from original not masked node
                ancest = (ptree+i)->abv;
                while(ancest < 2*nsam-2) {
                    ccnn = (ptree+ancest) -> ncs;
                    if(ccnn > 2) {
                        break;
                    }
                    if((ccnn == 2) && ((ptree+ancest)->masked == 0)) {   // if(ccnn == 2) {
                        y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                        (ptree+ancest)->masked = 1;

                        if( y >= x ) return( ancest ) ;
                    }
                    ancest = (ptree+ancest)->abv;
                }



            } else if(nc == (nsam-2)) {

                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                (ptree+i)->masked = 1;

                if( y >= x ) return( i ) ;
                // only start from original not masked node
                ancest = (ptree+i)->abv;
                while(ancest < 2*nsam-2) {
                    ccnn = (ptree+ancest) -> ncs;
                    if(ccnn > (nsam-2)) {
                        break;
                    }
                     if((ccnn == (nsam-2)) && ((ptree+ancest)->masked == 0)) {  //if(ccnn == (nsam-2)) {
                        y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                        (ptree+ancest)->masked = 1;

                        if( y >= x ) return( ancest ) ;
                    }
                    ancest = (ptree+ancest)->abv;
                }


            }
        }
   // } else {
    	/*for( i=nsam, y=0; i < 2*nsam-2 ; i++) {
            if((ptree+i)->ncs == 2) {
                if((ptree+i)->masked == 0) {
                    y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                    if( y >= x ) return( i ) ;
                    // only start from original not masked node
                    ancest = (ptree+i)->abv;
                    while(ancest < 2*nsam-2) {
                        ccnn = (ptree+ancest) -> ncs;
                        if(ccnn > 2) {
                            break;
                        }
                        if((ccnn == 2) && ((ptree+ancest)->masked == 0)) {
                            y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                            if( y >= x ) return( ancest ) ;
                        }
                        ancest = (ptree+ancest)->abv;
                    }
                }

            }
        }*/
   // }
	//printf("Warning: Have selected the LastNode: %d\n", lastNode);
	//return( lastNode );
}

int
pickb_OtonQM(nsam, ptree, tt)
	int nsam;
	struct node *ptree;
	double tt;
{
    double x, y=0.0, ran1();
	int i, j, nc, ccnn;
	int tip, ancest;
    x = ran1()*tt;

   // if(pars.mfs.unfolded != 1) {
         for( i=nsam, y=0; i < 2*nsam-2; i++) {
            if((ptree+i)->masked == 1) {
                continue;
            }
            nc = (ptree+i)->ncs;

            if((nc >= 3) && (nc != (nsam-1)) && (nc != (nsam-2))) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                (ptree+i)->masked = 1;

                if( y >= x ) return( i ) ;
                // only start from original not masked node
                ancest = (ptree+i)->abv;
                while(ancest < 2*nsam-2) {
                    ccnn = (ptree+ancest) -> ncs;
                    if((ccnn == (nsam-1)) || (ccnn == (nsam-2))) {
                        break;
                    }
                    if((ccnn >= 3) && (ccnn != (nsam-1)) && (ccnn != (nsam-2)) && ((ptree+ancest)->masked == 0)) {  // if((ccnn >= 3) && (ccnn != (nsam-1)) && (ccnn != (nsam-2))) {
                        y += (ptree + (ptree+ancest)->abv )->time - (ptree+ancest)->time;
                        (ptree+ancest)->masked = 1;

                        if( y >= x ) return( ancest ) ;
                    }
                    ancest = (ptree+ancest)->abv;
                }
            }

        }
   /* } else {
        for( i=nsam, y=0; i < 2*nsam-2; i++) {
            if((ptree+i)->ncs >=3 ) {
                y += (ptree + (ptree+i)->abv )->time - (ptree+i)->time;
                if( y >= x ) return( i ) ;
            }
        }
    } */


	//	printf("Warning: Have selected the node : %d\n", 2*nsam-3);
	return( 2*nsam-3 );
}

/****  tdesn : returns 1 if tip is a descendant of node in *ptree, otherwise 0. **/

	int
tdesn(struct node *ptree, int tip, int node )
{
	int k;

	for( k= tip ; k < node ; k = (ptree+k)->abv ) ;
	if( k==node ) return(1);
	else return(0);
}


/* pick2()  */

	int
pick2(int n, int *i, int *j)
{
	double ran1();

	*i = n * ran1() ;
	while( ( *j = n * ran1() ) == *i )
		;
	return(0) ;
}

/**** ordran.c  ***/

	void
ordran(int n,double pbuf[])
{
	ranvec(n,pbuf);
	order(n,pbuf);
	return;
}


	void
mnmial(int n, int nclass, double p[], int rv[])
{
	double ran1();
	double x, s;
	int i, j;

	for(i=0; i<nclass; i++) rv[i]=0;
	for(i=0; i<n ; i++) {
	   x = ran1();
	   j=0;
	   s = p[0];
	   while( (x > s) && ( j<(nclass-1) ) )  s += p[++j];
	   rv[j]++;
	   }
	return;
}

        void
order(int n,double pbuf[])
{
        int gap, i, j;
        double temp;

        for( gap= n/2; gap>0; gap /= 2)
           for( i=gap; i<n; i++)
                for( j=i-gap; j>=0 && pbuf[j]>pbuf[j+gap]; j -=gap) {
                   temp = pbuf[j];
                   pbuf[j] = pbuf[j+gap];
                   pbuf[j+gap] = temp;
                   }
    return;
}


	void
ranvec(int n,double pbuf[])
{
	int i;
	double ran1();

	for(i=0; i<n; i++)
		pbuf[i] = ran1();

	return;
}



	int
poisso(double u)
{
	double  cump, ru, ran1(), p, gasdev(double, double) ;
	int i=1;

	if( u > 30. ){
	    i =  (int)(0.5 + gasdev(u,u)) ;
	    if( i < 0 ) return( 0 ) ;
	    else return( i ) ;
	  }

	ru = ran1();
	p = exp(-u);
	if( ru < p) return(0);
	cump = p;

	while( ru > ( cump += (p *= u/i ) ) )
		i++;
	return(i);
}


/* a slight modification of crecipes version */

double gasdev(m,v)
	double m, v;
{
	static int iset=0;
	static float gset;
	float fac,r,v1,v2;
	double ran1();

	if  (iset == 0) {
		do {
			v1=2.0*ran1()-1.0;
			v2=2.0*ran1()-1.0;
			r=v1*v1+v2*v2;
		} while (r >= 1.0);
		fac=sqrt(-2.0*log(r)/r);
		gset= v1*fac;
		iset=1;
		return( m + sqrt(v)*v2*fac);
	} else {
		iset=0;
		return( m + sqrt(v)*gset ) ;
	}
}

